#define F_CPU 8000000UL  // 8 MHz

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>

#define byte unsigned char

#define ABS(a) ((a) > 0 ? (a) : (-(a)))

#define LED_DATA_BIT 			_BV(0)
#define LED_SHIFT_CLOCK_BIT 	_BV(2)
#define LED_REGISTER_CLOCK_BIT 	_BV(1)

#define LED_ROW_1				_BV(3)
#define LED_ROW_2				_BV(4)
#define LED_ROW_3				_BV(5)
#define LED_ROW_4				_BV(6)

#define ROWS					4
#define COLS					4
#define LEVELS					4	
#define COLORS					3

#define PIXELS					ROWS * COLS * LEVELS * 3
#define DISTLAY_SIZE			PIXELS / 8
#define LEVEL_SIZE				6

unsigned char display[DISTLAY_SIZE];

unsigned int timer30fps;

int counter;

void init(){
	//set port direction
	DDRD |= LED_DATA_BIT | LED_SHIFT_CLOCK_BIT | LED_REGISTER_CLOCK_BIT | LED_ROW_1 | LED_ROW_2 | LED_ROW_3 | LED_ROW_4;
	//clear port D
	PORTD = 0;
	//reset timer counter
	timer30fps = 0;
	counter = 0;
	//clear display
	for (int i = 0; i < DISTLAY_SIZE; i++){
		display[i]		= 0;
	}
}

const unsigned char dane[] = {0b10000000, 0b01000000, 0b00100000, 0b00010000, 0b00001000, 0b00000100, 0b00000010, 0b00000001};
void inline setBit(unsigned int p_byte, unsigned int p_bit){
	display[p_byte] |= dane[p_bit];
}

void inline putPixel(byte x, byte y, byte z, byte r, byte g, byte b){
	unsigned int index = (z + 1) * LEVEL_SIZE - 1;
	unsigned int bit_r = y * ROWS * 3 + x * 3;
	unsigned int bit_g = bit_r + 1;
	unsigned int bit_b = bit_g + 1;

	if (r){
		unsigned int index_r = index - bit_r / 8;
		setBit(index_r, bit_r % 8);
	}
	if (g){
		unsigned int index_g = index - bit_g / 8;
		setBit(index_g, bit_g % 8);
	}
	if (b){
		unsigned int index_b = index - bit_b / 8;
		setBit(index_b, bit_b % 8);
	}


}

void paintCube(byte x, byte y, byte z, byte r, byte g, byte b){
	putPixel(x, y, z, r, g, b);
	putPixel(x + 1, y, z, r, g, b);
	putPixel(x + 1, y + 1, z, r, g, b);
	putPixel(x, y + 1, z, r, g, b);
	putPixel(x, y, z + 1, r, g, b);
	putPixel(x + 1, y, z + 1, r, g, b);
	putPixel(x + 1, y + 1, z + 1, r, g, b);
	putPixel(x, y + 1, z + 1, r, g, b);
}

void drawLine(byte x1, byte y1, byte z1, byte x2, byte y2, byte z2, byte r, byte g, byte b){

	int dx = ABS(x2 - x1);
	int dy = ABS(y2 - y1);
	int dz = ABS(z2 - z1);

	if (dx >= dy && dx >= dz){
		dx = x2 - x1;
		dy = ((y2 - y1) * 256) / ABS(dx);
		dz = ((z2 - z1) * 256) / ABS(dx);
		int ddx = (dx / ABS(dx)) * 256;
		dx = ABS(dx);
		int x = x1 * 256;
		int y = y1 * 256;
		int z = z1 * 256;
		for (int i = 0; i < dx; i++){
			putPixel(x / 256, y / 256, z / 256, r, g, b);
			x += ddx;
			y += dy;
			z += dz;
		}
	}else
	if (dy >= dx && dy >= dz){
		dy = y2 - y1;
		dx = ((x2 - x1) * 256) / ABS(dy);
		dz = ((z2 - z1) * 256) / ABS(dy);
		int ddy = (dy / ABS(dy)) * 256;
		dy = ABS(dy);
		int x = x1 * 256;
		int y = y1 * 256;
		int z = z1 * 256;
		for (int i = 0; i < dy; i++){
			putPixel(x / 256, y / 256, z / 256, r, g, b);
			x += dx;
			y += ddy;
			z += dz;
		}
	}else
	if (dz >= dy && dz >= dx){
		dz = z2 - z1;
		dy = ((y2 - y1) * 256) / ABS(dz);
		dx = ((x2 - x1) * 256) / ABS(dz);
		int ddz = (dz / ABS(dz)) * 256;
		dz = ABS(dz);
		int x = x1 * 256;
		int y = y1 * 256;
		int z = z1 * 256;
		for (int i = 0; i < dz; i++){
			putPixel(x / 256, y / 256, z / 256, r, g, b);
			x += dx;
			y += dy;
			z += ddz;
		}
	}
	putPixel(x2, y2, z2, r, g, b);
}

void paintAnimation15(){
	static int level[4][4];
	static int sx;
	static int sy;
	if ((counter % 200) == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
		for (int x = 0; x < 4; x++){
			for (int y = 0; y < 4; y++){
				level[x][y] = 0;
			}
		}
	}
	if (counter % 2 == 0){
		if (level[sx][sy] < 3){
			putPixel(sx, sy, level[sx][sy], 1, 1, 1);
			level[sx][sy]++;
		}else{
			putPixel(sx, sy, level[sx][sy], 1, 1, 1);
			sx = rand() % 4;
			sy = rand() % 4;
		}
	}
}

void paintAnimation14(){
	if (counter % 2 == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
	
		int i = (counter / 2) % 8;
		if (i > 3) i = 7 - i;
	
		int step = counter / 30 + 1;
	
		unsigned char r = step & 0b001;
		unsigned char g = step & 0b010;
		unsigned char b = step & 0b100;

		if(!r && !g && !b){
			b = 1;
		}

		drawLine(0, i, 0, 0, 3 - i, 3, r, g, b);
		drawLine(3, 3 - i, 0, 3, i, 3, r, g, b);
		drawLine(0, i, 0, 3, 3 - i, 0, r, g, b);
		drawLine(0, 3 - i, 3, 3, i, 3, r, g, b);
	}	
}

void paintAnimation13(){
	static const int pattern[] = {0, 0, 3, 0,
								  0, 0, 3, 1,
								  0, 0, 3, 2,
								  0, 0, 2, 3,
								  0, 0, 1, 3,
								  0, 0, 0, 3,
								  1, 0, 0, 3,
								  2, 0, 0, 3,
								  3, 1, 0, 3,
								  3, 2, 0, 3,
								  3, 3, 0, 3,
								  3, 3, 0, 2,
								  3, 3, 0, 1,
								  3, 3, 1, 0,
								  3, 3, 2, 0,
								  3, 3, 3, 0,
								  2, 3, 3, 0,
								  1, 3, 3, 0,
								  0, 2, 3, 0,
								  0, 1, 3, 0,
								 };
	int direction = (counter / 100) % 3;
	if (counter % 2 == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
		int frame = (counter / 2) % 20;
		if (direction == 0){
			for (int z = 0; z < 4; z++){
				drawLine(z, pattern[frame << 2], pattern[(frame << 2) + 1], z, pattern[(frame << 2) + 2], pattern[(frame << 2) + 3], 1, 0, 0);
			}
		}
		if (direction == 1){
			for (int x = 0; x < 4; x++){
				drawLine(pattern[frame << 2], x, pattern[(frame << 2) + 1], pattern[(frame << 2) + 2], x, pattern[(frame << 2) + 3], 0, 1, 0);
			}
		}
		if (direction == 2){
			for (int y = 0; y < 4; y++){
				drawLine(pattern[frame << 2], pattern[(frame << 2) + 1], y, pattern[(frame << 2) + 2], pattern[(frame << 2) + 3], y, 0, 0, 1);
			}
		}
	}

}

void paintAnimation12(){
	int direction = (counter / 100) % 3;
	static const int pattern[] = {0, 0, 
								  1, 0, 
								  2, 0,
								  3, 0,
								  3, 1,
								  3, 2,
								  3, 3,
								  2, 3,
								  1, 3,
								  0, 3,
								  0, 2,
								  0, 1,
								 };
	
	if (counter % 2 == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
		int frame1 = (counter / 2) % 12;
		int frame2 = (counter / 2 + 6) % 12;
		if (direction == 0) drawLine(pattern[frame1 << 1], pattern[(frame1 << 1) + 1], 0, pattern[frame2 << 1], pattern[(frame2 << 1) + 1], 3, 1, 0, 0);
		if (direction == 1) drawLine(0, pattern[frame1 << 1], pattern[(frame1 << 1) + 1], 3, pattern[frame2 << 1], pattern[(frame2 << 1) + 1], 0, 1, 0);
		if (direction == 2) drawLine(pattern[frame1 << 1], 0, pattern[(frame1 << 1) + 1], pattern[frame2 << 1], 3, pattern[(frame2 << 1) + 1], 0, 0, 1);
	}
	
}

void paintAnimation11(){
	if (counter % 2 == 0){
		if (counter % 12 < 6){
			for (int i = 0; i < DISTLAY_SIZE; i++){
				display[i] = 0;
			}
		}else{
			for (int i = 0; i < DISTLAY_SIZE; i++){
				display[i] = rand();
			}
		}
	}	
}

void paintAnimation10(){
	static int level[4][4];
	static int sx;
	static int sy;
	static int dir;
	if (counter % 2 == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
		if (level[sx][sy] == 3 && dir == 0){
			sx = rand() % 4;
			sy = rand() % 4;
			dir = 1;
		}else if (level[sx][sy] == 0 && dir == 1){
			sx = rand() % 4;
			sy = rand() % 4;
			dir = 0;
		}else{
			if (dir) level[sx][sy]--;
			else level[sx][sy]++;
		}

		for (int x = 0; x < 4; x++){
			for (int y = 0; y < 4; y++){
				putPixel(x, y, level[x][y], 1, 1, 1);
			}
		}
	}
}

void paintAnimation9(){
	static int level;
	static int dl;
	int direction = (counter / 100) % 3;

	int step = counter / 30 + 1;
	
	unsigned char r = step & 0b001;
	unsigned char g = step & 0b010;
	unsigned char b = step & 0b100;

	if(!r && !g && !b){
		b = 1;
	}

	if (counter % 600 == 0){
		dl = 1;
		level = 0;
	}
	if (counter % 3 == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
		for (int x = 0; x < 4; x++){
			for (int y = 0; y < 4; y++){
				if (direction == 0) putPixel(x, y, level, r, g, b);
				if (direction == 1) putPixel(x, level, y, r, g, b);
				if (direction == 2) putPixel(level, y, x, r, g, b);
			}
		}
		level += dl;
		if (level == 3) dl = -1;
		else if (level == 0) dl = 1;
	}
}

void paintAnimation8(){
	const unsigned char pattern[] = {0,0,
									 0,0,
									 0,0,
									 0,0,
									 1,0,
									 2,0,
									 2,0,
									 2,0,
									 2,0,
									 2,1,
									 2,2,
									 2,2,
									 2,2,
									 2,2,
									 1,2,
									 0,2,
									 0,2,
									 0,2,
									 0,2,
									 0,1,
									 };
	static int frame;

	if (counter % 3 == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}


		for (int x = 0; x < 8; x += 2){
			int step = ((counter / 4) + x / 2) / 6 + 1;
			unsigned char r = step & 0b001;
			unsigned char g = step & 0b010;
			unsigned char b = step & 0b100;
			if(!r && !g && !b){
				b = 1;
			}
			putPixel(x >> 1, pattern[(frame + x) % 40], 		pattern[(frame + x + 1) % 40], r, g, b);
			putPixel(x >> 1, pattern[(frame + x) % 40], 		pattern[(frame + x + 1) % 40] + 1, r, g, b);
			putPixel(x >> 1, pattern[(frame + x) % 40] + 1, 	pattern[(frame + x + 1) % 40] + 1, r, g, b);
			putPixel(x >> 1, pattern[(frame + x) % 40] + 1, 	pattern[(frame + x + 1) % 40], r, g, b);
		}
		
		frame = (frame + 2) % 40;
	}
}


void paintAnimation7(){
	const unsigned char pattern[] = {0,0,0,
									 1,0,0,
									 2,0,0,
									 2,1,0,
									 2,2,0,
									 2,2,1,
									 2,2,2,
									 1,2,2,
									 0,2,2,
									 0,1,2,
									 0,0,2,
									 0,0,1
									 };
	static int frame;

	if (counter % 6 == 0){
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
		
		paintCube(pattern[frame], pattern[frame + 1], pattern[frame + 2], 1, 0, 0);		
		paintCube(2 - pattern[frame], 2 - pattern[frame + 1], 2 - pattern[frame + 2], 0, 0, 1);		
		frame = (frame + 3) % 36;
	}
										
}


void paintAnimation6(){
	static int y[4];
	static int direction;

	if (direction == 0) direction = 1;	
	if (counter % 6 == 0){
		for (int i = 3; i >= 1; i--){
			y[i] = y[i - 1];
		}
		y[0] += direction;
		if (y[0] == 0 || y[0] == 3) direction = - direction;
	}

	for (int i = 0; i < DISTLAY_SIZE; i++){
		display[i] = 0;
	}

	int step = counter / 30 + 1;
	
	unsigned char r = step & 0b001;
	unsigned char g = step & 0b010;
	unsigned char b = step & 0b100;

	if(!r && !g && !b){
		b = 1;
	}

	for (int xp = 0; xp < 4; xp++){
		for (int yp = 0; yp < 4; yp++){
				putPixel(xp, yp, y[yp], r, g, b);
		}
	}


}

void paintAnimation5(){
	const unsigned char pattern[] = {0,0,0,1,
									 0,0,1,0,
									 0,1,0,0,
									 1,0,0,0,

									 0,0,0,0,
									 0,0,1,1,
									 1,1,0,0,
									 0,0,0,0,

									 0,0,0,0,
									 1,1,0,0,
									 0,0,1,1,
									 0,0,0,0,

									 1,0,0,0,
									 0,1,0,0,
									 0,0,1,0,
									 0,0,0,1,

									 0,1,0,0,
									 0,1,0,0,
									 0,0,1,0,
									 0,0,1,0,

									 0,0,1,0,
									 0,0,1,0,
									 0,1,0,0,
									 0,1,0,0,
									}; 

	static int frame;
	if (counter % 4 == 0){

		int step = counter / 30 + 1;
	
		unsigned char r = step & 0b001;
		unsigned char g = step & 0b010;
		unsigned char b = step & 0b100;

		if(!r && !g && !b){
			b = 1;
		}

		int index = frame * 16;
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
		for (int x = 0; x < COLS; x++){
			for (int y = 0; y < ROWS; y++){
				if (pattern[index]) {
					for (int z = 0; z < LEVELS; z++){
						putPixel(x, y, z, r, g, b);
					}
				}
				index++;
			}
		}
		frame++;
		if (frame > 5) {
			frame = 0;
		}
	}

}


void paintAnimation4(){
	const unsigned char patterns[] = {1, 0, 0, 1, 0, 0,/**/0, 1, 0, 0, 1, 0,/**/1, 0, 0, 1, 0, 0,/**/0, 1, 0, 0, 1, 0,/**/1, 0, 0, 1, 0, 0,/**/0, 1, 0, 0, 1, 0,/**/ };
	const unsigned char shape[] = {0,0,0,1,0,2,0,3,1,3,2,3,3,3,3,2,3,1,3,0,2,0,1,0};
	
	static unsigned char rl[12];
	static unsigned char gl[12];
	static unsigned char bl[12];
	if(counter % 600 == 0){
		int index = 0;
		for (int i = 0; i < 12 * 3;){
			rl[index] = patterns[i++];
			gl[index] = patterns[i++];
			bl[index++] = patterns[i++];
		}
	}
	unsigned char rt = rl[0];
	unsigned char gt = gl[0];
	unsigned char bt = bl[0];
	if (counter % 5 == 0){
		for (int i = 0; i < 11; i++){
			rl[i] = rl[i + 1];
			gl[i] = gl[i + 1];
			bl[i] = bl[i + 1];
		}
		rl[11] = rt;
		gl[11] = gt;
		bl[11] = bt;
	}

	for (int i = 0; i < DISTLAY_SIZE; i++){
		display[i] = 0;
	}
	for (int i = 0; i < 12; i++){
		putPixel(shape[i << 1], shape[(i << 1) + 1], 0, rl[i], gl[i], bl[i]);
		putPixel(shape[i << 1], shape[(i << 1) + 1], 1, rl[i], gl[i], bl[i]);
		putPixel(shape[i << 1], shape[(i << 1) + 1], 2, rl[i], gl[i], bl[i]);
		putPixel(shape[i << 1], shape[(i << 1) + 1], 3, rl[i], gl[i], bl[i]);
	}
	if (counter % 4 > 1){
		paintCube(1,1,0,0,0,1);
		paintCube(1,1,2,0,0,1);
	}else{
		paintCube(1,1,0,1,1,0);
		paintCube(1,1,2,1,1,0);
	}
}






void paintAnimation3(){
	static int cx[3];
	static int cy[3];
	static int cz[3];
	static int dx[3];
	static int dy[3];
	static int dz[3];
	static unsigned char clear;


	if (!clear){
		for (int i = 0; i < 3; i++){
			cx[i] = 1;
			cy[i] = 1;
			cz[i] = 1;
			
			dx[i] = (rand() % 3) - 1;
			dy[i] = (rand() % 3) - 1;
			dz[i] = (rand() % 3) - 1;
		}
		clear = 1;
	}

	int step = counter % 16;
	if (step == 0){
	for (int i = 0; i < 3; i++){
		cx[i] += dx[i];
		cy[i] += dy[i];
		cz[i] += dz[i];


		if (cx[i] == 0){
			dx[i] = rand() % 2;
		}else if (cx[i] == 2){
			dx[i] = (rand() % 2) - 1;
		}
		if (cy[i] == 0){
			dy[i] = rand() % 2;
		}else if (cy[i] == 2){
			dy[i] = (rand() % 2) - 1;
		}
		if (cz[i] == 0){
			dz[i] = rand() % 2;
		}else if (cz[i] == 2){
			dz[i] = (rand() % 2) - 1;
		}
	}
	}

	for (int i = 0; i < DISTLAY_SIZE; i++){
		display[i] = 0;
	}
	paintCube(cx[0], cy[0], cz[0], 1, 0, 0);
	//paintCube(cx[1], cy[1], cz[1], 0, 1, 0);
	//paintCube(cx[2], cy[2], cz[2], 0, 0, 1);
}

void paintAnimation2(){
	static unsigned char clear;
	if (!clear){
		clear = 1;
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
		}
	}
	putPixel(rand() % 4, rand() % 4, rand() % 4, rand() % 2, rand() % 2, rand() % 2);
	if (counter % 100 == 0) clear = 0;
}


void paintAnimation1(){
	int step = counter / 30 + 1;
	
	if (counter % 30 == 0)
		for (int i = 0; i < DISTLAY_SIZE; i++){
			display[i] = 0;
	}
	
	
	unsigned char r = step & 0b001;
	unsigned char g = step & 0b010;
	unsigned char b = step & 0b100;


	//if (r)
	//for (int i = 0; i < DISTLAY_SIZE; i++){
	//	display[i] = 255;
	//}

	if(!r && !g && !b){
		b = 1;
	}

	for (int x = 0; x < COLS; x++){
		for (int y = 0; y < ROWS; y++){
			for (int z = 0; z < LEVELS; z++){
				putPixel(x, y, ((counter % 30) / 3) % 4, r, g, b);
			}
		}
	}
	
	
}


void paintFrame(){
	int step = counter / 600;
	if (step == 14){
		counter = 0;
	}
	if (step == 0){
		paintAnimation15();
	}
	if (step == 13){
		paintAnimation14();
	}
	if (step == 12){
		paintAnimation13();
	}
	if (step == 11){
		paintAnimation12();
	}
	if (step == 10){
		paintAnimation9();
	}
	if (step == 9){
		paintAnimation11();
	}
	if (step == 8){
		paintAnimation10();
	}
	if (step == 1){
		paintAnimation8();
	}
	if (step == 2){
		paintAnimation7();
	}
	if (step == 3){
		paintAnimation6();
	}
	if (step == 4){
		paintAnimation5();
	}
	if (step == 5){
		paintAnimation4();
	}
	if (step == 6){
		paintAnimation2();
	}
	if (step == 7){
		paintAnimation1();
	}
	counter++;
}

void sendByte(unsigned char c){
	//send serial data for every bit
	for (unsigned char i = 0; i < 8; i++){
		if (c & 1){
			PORTD |= LED_DATA_BIT;
		}else{
			PORTD &= ~LED_DATA_BIT;
		}
		c = (unsigned char)c >> 1;
		PORTD |= LED_SHIFT_CLOCK_BIT;
		_delay_us(1);
		PORTD &= ~LED_SHIFT_CLOCK_BIT;
		_delay_us(1);
	}
}

void latchData(){
	//latch data on D register
	PORTD |= LED_REGISTER_CLOCK_BIT;
	_delay_us(1);
	PORTD &= ~LED_REGISTER_CLOCK_BIT;
	_delay_us(1);
}

void sendLevel(int l){
	//send data for one level
	int index = l * LEVEL_SIZE;
	for (int i = 0; i < LEVEL_SIZE; i++){
		sendByte(display[index + i]);
	}
	latchData();
}

int main() {
	//init 
	init();
	//set light on!!!

	

	while(1){
		sendLevel(0);
		PORTD |= LED_ROW_1;
		_delay_us(1000);
		PORTD &= ~LED_ROW_1;
		sendLevel(1);
		PORTD |= LED_ROW_2;
		_delay_us(1000);
		PORTD &= ~LED_ROW_2;
		sendLevel(2);
		PORTD |= LED_ROW_3;
		_delay_us(1000);
		PORTD &= ~LED_ROW_3;
		sendLevel(3);
		PORTD |= LED_ROW_4;
		_delay_us(1000);
		PORTD &= ~LED_ROW_4;
		timer30fps++;
		if (timer30fps > 5){
			timer30fps = 0;
			paintFrame();
		}
	}
	return 0;
}




